<?php

namespace Drupal\javali_spreadsheets;

use Google_Client;
use Google_Service_Sheets;

/**
 * Class for the service  javali_sheets.base .
 *
 * This service enables the creation,  * configuration and data
 * retrieval of a google sheet.
 *
 * @author kallado
 */
class JavaliSheets {

  private $client;
  private $sheetId;
  private $range;
  private $sheetName;

  /**
   * JavaliSheets constructor.
   */
  public function __construct() {
    $config = \Drupal::config('javali.google_sheets_config');
    $this->client = $this->getClient();
    $this->sheet_id = $config->get('sheet_id');
    $this->range = $config->get('range');
    $this->sheetName = $config->get('sheet_name');
  }

  /**
   * Function to get the Google client.
   *
   * @return \Google_Client
   *   the Google client
   *
   * @throws Exception
   */
  public function getClient() {
    $client = new Google_Client();
    $client->setApplicationName('Google Sheets API PHP Quickstart');
    $client->setScopes(Google_Service_Sheets::SPREADSHEETS_READONLY);
    $client->setAuthConfig('credentials.json');
    $client->setAccessType('offline');

    // Load previously authorized credentials from a file.
    $credentialsPath = 'token.json';
    if (file_exists($credentialsPath)) {
      $accessToken = json_decode(file_get_contents($credentialsPath), TRUE);
    }
    else {
      // Request authorization from the user.
      $authUrl = $client->createAuthUrl();
      printf("Open the following link in your browser:\n%s\n", $authUrl);
      print 'Enter verification code: ';
      $authCode = trim(fgets(STDIN));

      // Exchange authorization code for an access token.
      $accessToken = $client->fetchAccessTokenWithAuthCode($authCode);

      // Check to see if there was an error.
      if (array_key_exists('error', $accessToken)) {
        throw new Exception(implode(', ', $accessToken));
      }

      // Store the credentials to disk.
      if (!file_exists(dirname($credentialsPath))) {
        mkdir(dirname($credentialsPath), 0700, TRUE);
      }
      file_put_contents($credentialsPath, json_encode($accessToken));
      printf("Credentials saved to %s\n", $credentialsPath);
    }
    $client->setAccessToken($accessToken);

    // Refresh the token if it's expired.
    if ($client->isAccessTokenExpired()) {
      $client->fetchAccessTokenWithRefreshToken($client->getRefreshToken());
      file_put_contents($credentialsPath, json_encode($client->getAccessToken()));
    }
    return $client;
  }

  /**
   * Get the API client and construct the service object.
   *
   * @return array
   *   an array with the sheet data
   */
  public function getdata() {
    $client = $this->client;
    $service = new Google_Service_Sheets($client);
    $spreadsheetId = $this->sheet_id;
    $name = $this->sheetName;
    $range = $this->range;

    try {
      $sheetInfo = $service->spreadsheets->get($spreadsheetId);
    }
    catch (\Google_Service_Exception $e) {
      $error = json_decode($e->getMessage(), TRUE);
      return drupal_set_message($error['error']['message'], 'error');
    }

    $allsheet_info = $sheetInfo['sheets'];
    $idCats = array_column($allsheet_info, 'properties');

    // Checks if the sheet exists.
    if (!$this->myArrayContainsWord($idCats, $name)) {
      return drupal_set_message('Sheet not found', 'error');
    }

    $range_format = $name . '!' . $range;

    try {
      $response = $service->spreadsheets_values->get($spreadsheetId, $range_format);
      return $response;
    }
    catch (\Google_Service_Exception $e) {
      $error = json_decode($e->getMessage(), TRUE);
      return drupal_set_message($error['error']['message'], 'error');
    }

  }

  /**
   * Set the sheet id.
   *
   * @param string $id
   *   The google sheet id.
   */
  public function setSheetId(string $id) {
    $this->sheet_id = $id;
  }

  /**
   * Set the string range.
   *
   * @param string $range
   *   range (format ex A1:B2)
   */
  public function setRange(string $range) {
    $this->range = $range;
  }

  /**
   * Set the sheet name.
   *
   * @param string $name
   *   The name of the shee (ex:Sheet1)
   */
  public function setSheetName(string $name) {
    $this->sheetName = $name;
  }

  /**
   * Checks if myArray contains a word as key.
   *
   * @param array $myArray
   *   The array to check.
   * @param mixed $word
   *   The word to find.
   *
   * @return bool
   *   True if the word is found false otherwise
   */
  public function myArrayContainsWord(array $myArray, $word) {
    foreach ($myArray as $element) {
      if ($element->title == $word) {
        return TRUE;
      }
    }
    return FALSE;
  }

}
